@@ -16,13 +16,18 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <fcntl.h>
+#include <unistd.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <glib.h>
 #include "fileops.h"
+#include "fileops-helper.h"
 #include "functions.h"
 #include "comiclist.h"
+#include "debug.h"
 
 const gchar *CO19Ident     = "CO19_File";
 const gchar *GCO_0_1_Ident = "GCO_0_1__";
@@ -35,115 +40,115 @@ GList *LoadCO19(GList *ComicList, FILE *fp)
   TIssue *TempIssue;
   GString * TempString;
 
-  while((c=getc(fp))!=EOF)
+  while((c=my_getc())!=EOF)
     {
-      ungetc(c, fp);
+      my_ungetc (c);
       TempComic = Init_Comic();
 
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CTitle = g_string_append_c(TempComic->CTitle, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CPublisher = g_string_append_c(TempComic->CPublisher, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CType = g_string_append_c(TempComic->CType, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CVolume = g_string_append_c(TempComic->CVolume, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CGroup = g_string_append_c(TempComic->CGroup, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->Comment = g_string_append_c(TempComic->Comment, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->LinkFile = g_string_append_c(TempComic->LinkFile, c);
 	}
       TempComic->IsLink = (TempComic->LinkFile->len>0);
 
       /* Now load the Issues */
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
-	  ungetc(c,fp);
+	  my_ungetc (c);
 
 	  TempIssue = Init_Issue();
 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->IssueNo = g_string_append_c(TempIssue->IssueNo, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Writer = g_list_append(TempIssue->Writer, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Penciller = g_list_append(TempIssue->Penciller, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Inker = g_list_append(TempIssue->Inker, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Title = g_list_append(TempIssue->Title, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Location = g_string_append_c(TempIssue->Location, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Price = g_string_append_c(TempIssue->Price, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->CurrentPrice = g_string_append_c(TempIssue->CurrentPrice, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Image = g_string_append_c(TempIssue->Image, c);
   	    } 
 
 	  TempString = g_string_new(NULL);
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempString = g_string_append_c(TempString, c);
   	    }
@@ -167,11 +172,11 @@ GList *LoadCO19(GList *ComicList, FILE *fp)
 	    TempIssue->Condition = 8;
 
 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Comment = g_string_append_c(TempIssue->Comment, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      /*
 	      TempIssue->Date = g_string_append_c(TempIssue->Date, c);
@@ -203,49 +208,49 @@ GList *LoadGCO_0_1(GList *ComicList, FILE *fp)
   GString * TempString;
 
   TempString = g_string_new(NULL);
-  while((c=getc(fp))!='\x00')
+  while((c=my_getc())!='\x00')
     {
       TempString = g_string_append_c(TempString, c);
     }
   LargestUID = atof(TempString->str);
   g_string_free(TempString, TRUE);
 
-  while((c=getc(fp))!=EOF)
+  while((c=my_getc())!=EOF)
     {
-      ungetc(c, fp);
+      my_ungetc (c);
       TempComic = Init_Comic();
 
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CTitle = g_string_append_c(TempComic->CTitle, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CPublisher = g_string_append_c(TempComic->CPublisher, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CType = g_string_append_c(TempComic->CType, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CVolume = g_string_append_c(TempComic->CVolume, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CGroup = g_string_append_c(TempComic->CGroup, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->Comment = g_string_append_c(TempComic->Comment, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->LinkFile = g_string_append_c(TempComic->LinkFile, c);
 	}
       TempComic->IsLink = (TempComic->LinkFile->len>0);
       TempString = g_string_new(NULL);
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempString = g_string_append_c(TempString, c);
 	}
@@ -253,95 +258,95 @@ GList *LoadGCO_0_1(GList *ComicList, FILE *fp)
       g_string_free(TempString, TRUE);
 
       /* Now load the Issues */
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
-	  ungetc(c,fp);
+	  my_ungetc (c);
 	  TempIssue = Init_Issue();
 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->IssueNo = g_string_append_c(TempIssue->IssueNo, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Writer = g_list_append(TempIssue->Writer, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Penciller = g_list_append(TempIssue->Penciller, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Inker = g_list_append(TempIssue->Inker, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Title = g_list_append(TempIssue->Title, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Location = g_string_append_c(TempIssue->Location, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Price = g_string_append_c(TempIssue->Price, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->CurrentPrice = g_string_append_c(TempIssue->CurrentPrice, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Image = g_string_append_c(TempIssue->Image, c);
   	    } 
 	  TempString = g_string_new(NULL);
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempString = g_string_append_c(TempString, c);
   	    }
 	  TempIssue->Condition = atof(TempString->str);
 	  g_string_free(TempString, TRUE);
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Comment = g_string_append_c(TempIssue->Comment, c);
   	    } 
 	  TempString = g_string_new(NULL);
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempString = g_string_append_c(TempString, c);
   	    }
 	  TempIssue->Date = atof(TempString->str);
 	  g_string_free(TempString, TRUE);
 	  TempString = g_string_new(NULL);
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempString = g_string_append_c(TempString, c);
   	    }
@@ -371,49 +376,49 @@ GList *LoadGCO_0_3(GList *ComicList, FILE *fp)
   GString * TempString;
 
   TempString = g_string_new(NULL);
-  while((c=getc(fp))!='\x00')
+  while((c=my_getc())!='\x00')
     {
       TempString = g_string_append_c(TempString, c);
     }
   LargestUID = atof(TempString->str);
   g_string_free(TempString, TRUE);
 
-  while((c=getc(fp))!=EOF)
+  while((c=my_getc())!=EOF)
     {
-      ungetc(c, fp);
+      my_ungetc (c);
       TempComic = Init_Comic();
 
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CTitle = g_string_append_c(TempComic->CTitle, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CPublisher = g_string_append_c(TempComic->CPublisher, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CType = g_string_append_c(TempComic->CType, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CVolume = g_string_append_c(TempComic->CVolume, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->CGroup = g_string_append_c(TempComic->CGroup, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->Comment = g_string_append_c(TempComic->Comment, c);
 	}
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempComic->LinkFile = g_string_append_c(TempComic->LinkFile, c);
 	}
       TempComic->IsLink = (TempComic->LinkFile->len>0);
       TempString = g_string_new(NULL);
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
 	  TempString = g_string_append_c(TempString, c);
 	}
@@ -421,99 +426,99 @@ GList *LoadGCO_0_3(GList *ComicList, FILE *fp)
       g_string_free(TempString, TRUE);
 
       /* Now load the Issues */
-      while((c=getc(fp))!='\x00')
+      while((c=my_getc())!='\x00')
 	{
-	  ungetc(c,fp);
+	  my_ungetc (c);
 	  TempIssue = Init_Issue();
 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->IssueNo = g_string_append_c(TempIssue->IssueNo, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Writer = g_list_append(TempIssue->Writer, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Penciller = g_list_append(TempIssue->Penciller, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Inker = g_list_append(TempIssue->Inker, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
-	      ungetc(c,fp);
+	      my_ungetc (c);
 	      TempString = g_string_new(NULL);
 
-	      while((c=getc(fp))!='\x02')
+	      while((c=my_getc())!='\x02')
 		{
 		  TempString = g_string_append_c(TempString, c);
 		}
 	      TempIssue->Title = g_list_append(TempIssue->Title, TempString);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Location = g_string_append_c(TempIssue->Location, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Price = g_string_append_c(TempIssue->Price, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->CurrentPrice = g_string_append_c(TempIssue->CurrentPrice, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->PurchasePrice = g_string_append_c(TempIssue->PurchasePrice, c);
   	    } 
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Image = g_string_append_c(TempIssue->Image, c);
   	    } 
 	  TempString = g_string_new(NULL);
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempString = g_string_append_c(TempString, c);
   	    }
 	  TempIssue->Condition = atof(TempString->str);
 	  g_string_free(TempString, TRUE);
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempIssue->Comment = g_string_append_c(TempIssue->Comment, c);
   	    } 
 	  TempString = g_string_new(NULL);
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempString = g_string_append_c(TempString, c);
   	    }
 	  TempIssue->Date = atof(TempString->str);
 	  g_string_free(TempString, TRUE);
 	  TempString = g_string_new(NULL);
-  	  while((c=getc(fp))!='\x01')
+  	  while((c=my_getc())!='\x01')
   	    { 
 	      TempString = g_string_append_c(TempString, c);
   	    }
@@ -546,10 +551,10 @@ GList *LoadFile(GList *ComicList, FILE *fp)
       return LoadGCO_0_3(ComicList, fp);
       break;
     case NotCO: 
-      g_print("File is not a recognized filetype...\n");
+      dprintf("File is not a recognized filetype...\n");
       return NULL;
     default:
-      g_print("Something unexpected happened...\n");
+      dprintf("Something unexpected happened...\n");
       return NULL;
     }
 }
@@ -564,98 +569,98 @@ void SaveFile(GList *ComicList, FILE *fp)
   TIssue *TempIssue;
   GString * TempString;
 
-  fputc('\x00', fp);
-  fputs(GCO_0_3_Ident, fp);
-  fputc('\x00', fp);
-  fputs(itoa(GetUID()+1), fp); 
-  fputc('\x00', fp);
+  my_putc('\x00', fp);
+  my_puts(GCO_0_3_Ident, fp);
+  my_putc('\x00', fp);
+  my_puts(itoa(GetUID()+1), fp); 
+  my_putc('\x00', fp);
 
   j = g_list_length(ComicList);
   for (i=0;i<j;i++)
     {
       /* Save the Comic info */
       TempComic = (g_list_nth(ComicList,i))->data;
-      fputs(TempComic->CTitle->str, fp);
-      fputc('\x00', fp);
-      fputs(TempComic->CPublisher->str, fp);
-      fputc('\x00', fp);
-      fputs(TempComic->CType->str, fp);
-      fputc('\x00', fp);
-      fputs(TempComic->CVolume->str, fp);
-      fputc('\x00', fp);
-      fputs(TempComic->CGroup->str, fp);
-      fputc('\x00', fp);
-      fputs(TempComic->Comment->str, fp);
-      fputc('\x00', fp);
-      fputs(TempComic->LinkFile->str, fp);
-      fputc('\x00', fp);
-      fputs(itoa(TempComic->UID), fp); 
-      fputc('\x00', fp);
+      my_puts(TempComic->CTitle->str, fp);
+      my_putc('\x00', fp);
+      my_puts(TempComic->CPublisher->str, fp);
+      my_putc('\x00', fp);
+      my_puts(TempComic->CType->str, fp);
+      my_putc('\x00', fp);
+      my_puts(TempComic->CVolume->str, fp);
+      my_putc('\x00', fp);
+      my_puts(TempComic->CGroup->str, fp);
+      my_putc('\x00', fp);
+      my_puts(TempComic->Comment->str, fp);
+      my_putc('\x00', fp);
+      my_puts(TempComic->LinkFile->str, fp);
+      my_putc('\x00', fp);
+      my_puts(itoa(TempComic->UID), fp); 
+      my_putc('\x00', fp);
 
       /* Now save the Issues */
       k = g_list_length(TempComic->IssueNumbers);
       for (l=0;l<k;l++)
 	{
 	  TempIssue = (g_list_nth(TempComic->IssueNumbers,l))->data;
-	  fputs(TempIssue->IssueNo->str, fp);
-	  fputc('\x01', fp);
+	  my_puts(TempIssue->IssueNo->str, fp);
+	  my_putc('\x01', fp);
 
 	  m = g_list_length(TempIssue->Writer);
 	  for (n=0;n<m;n++)
 	    {
 	      TempString = (g_list_nth(TempIssue->Writer, n)->data);
-	      fputs(TempString->str, fp);
-	      fputc('\x02', fp);
+	      my_puts(TempString->str, fp);
+	      my_putc('\x02', fp);
 	    }
-	  fputc('\x01', fp);
+	  my_putc('\x01', fp);
 
 	  m = g_list_length(TempIssue->Penciller);
 	  for (n=0;n<m;n++)
 	    {
 	      TempString = (g_list_nth(TempIssue->Penciller, n)->data);
-	      fputs(TempString->str, fp);
-	      fputc('\x02', fp);
+	      my_puts(TempString->str, fp);
+	      my_putc('\x02', fp);
 	    }
-	  fputc('\x01', fp);
+	  my_putc('\x01', fp);
 
 	  m = g_list_length(TempIssue->Inker);
 	  for (n=0;n<m;n++)
 	    {
 	      TempString = (g_list_nth(TempIssue->Inker, n)->data);
-	      fputs(TempString->str, fp);
-	      fputc('\x02', fp);
+	      my_puts(TempString->str, fp);
+	      my_putc('\x02', fp);
 	    }
-	  fputc('\x01', fp);
+	  my_putc('\x01', fp);
 
 	  m = g_list_length(TempIssue->Title);
 	  for (n=0;n<m;n++)
 	    {
 	      TempString = (g_list_nth(TempIssue->Title, n)->data);
-	      fputs(TempString->str, fp);
-	      fputc('\x02', fp);
+	      my_puts(TempString->str, fp);
+	      my_putc('\x02', fp);
 	    }
-	  fputc('\x01', fp);
+	  my_putc('\x01', fp);
 
-	  fputs(TempIssue->Location->str, fp);
-	  fputc('\x01', fp);
-	  fputs(TempIssue->Price->str, fp);
-	  fputc('\x01', fp);
-	  fputs(TempIssue->CurrentPrice->str, fp);
-	  fputc('\x01', fp);
-	  fputs(TempIssue->PurchasePrice->str, fp);
-	  fputc('\x01', fp);
-	  fputs(TempIssue->Image->str, fp);
-	  fputc('\x01', fp);
-	  fputs(itoa(TempIssue->Condition), fp);
-	  fputc('\x01', fp);
-	  fputs(TempIssue->Comment->str, fp);
-	  fputc('\x01', fp);
-	  fputs(itoa(TempIssue->Date), fp);
-	  fputc('\x01', fp);
-	  fputs(itoa(TempIssue->UID), fp);
-	  fputc('\x01', fp);
+	  my_puts(TempIssue->Location->str, fp);
+	  my_putc('\x01', fp);
+	  my_puts(TempIssue->Price->str, fp);
+	  my_putc('\x01', fp);
+	  my_puts(TempIssue->CurrentPrice->str, fp);
+	  my_putc('\x01', fp);
+	  my_puts(TempIssue->PurchasePrice->str, fp);
+	  my_putc('\x01', fp);
+	  my_puts(TempIssue->Image->str, fp);
+	  my_putc('\x01', fp);
+	  my_puts(itoa(TempIssue->Condition), fp);
+	  my_putc('\x01', fp);
+	  my_puts(TempIssue->Comment->str, fp);
+	  my_putc('\x01', fp);
+	  my_puts(itoa(TempIssue->Date), fp);
+	  my_putc('\x01', fp);
+	  my_puts(itoa(TempIssue->UID), fp);
+	  my_putc('\x01', fp);
 	}
-      fputc('\x00', fp);
+      my_putc('\x00', fp);
     }
 }
 
@@ -669,7 +674,7 @@ TFileType FileCheck(FILE *fp)
   s = g_string_new(NULL);
   for(i=0;i<=10;i++)
     {
-      if ((c=getc(fp))==EOF)
+      if ((c=my_getc())==EOF)
 	g_print("Error...\n");
       if (c!=0) /* Idents are surrounded by #0's */
 	s = g_string_append_c(s, c);
@@ -685,12 +690,20 @@ TFileType FileCheck(FILE *fp)
     return NotCO;
 }
 
-void ExportToASCIIFile(GList *clist, FILE *fp)
+int ExportToASCIIFile(GList *clist, GString *fname)
 {
   TComic *TempComic;
   TIssue *TempIssue;
   guint32 i,j,k,l,m,n;
   GString *String;
+  FILE *fp;
+
+  fp = fopen (fname->str, "w");
+  if (fp == NULL)
+    {
+      /* Unable to open file */
+      return 1;
+    }
 
   fprintf(fp, "%i comic title(s)\n\n\n\n", j=g_list_length(clist));
   for(i=0;i<j;i++)
@@ -798,4 +811,509 @@ void ExportToASCIIFile(GList *clist, FILE *fp)
 	}
       fprintf(fp, "\n\n");
     }
+  
+  fclose(fp);
+  return 0;
+}
+
+void write_html_preamble (FILE *fp, char *title)
+{
+  fprintf (fp, "<html>");
+  fprintf (fp, "<title>");
+  fprintf (fp, title);
+  fprintf (fp, "</title>");
+  fprintf (fp, "<body bgcolor=\"white\">");
+}
+
+void write_html_end (FILE *fp)
+{
+  fprintf (fp, "</body>");
+  fprintf (fp, "</html>");
+}
+
+void write_html_h1_start (FILE *fp)
+{
+  fprintf (fp, "<h1>");
+}
+
+void write_html_h1_end (FILE *fp)
+{
+  fprintf (fp, "</h1>");
+}
+
+void write_html_h2_start (FILE *fp)
+{
+  fprintf (fp, "<h2>");
+}
+
+void write_html_h2_end (FILE *fp)
+{
+  fprintf (fp, "</h2>");
+}
+
+void write_html_h3_start (FILE *fp)
+{
+  fprintf (fp, "<h3>");
+}
+
+void write_html_h3_end (FILE *fp)
+{
+  fprintf (fp, "</h3>");
+}
+
+void write_html_br (FILE *fp)
+{
+  fprintf (fp, "<br>");
+}
+
+void write_html_p (FILE *fp)
+{
+  fprintf (fp, "<p>");
+}
+
+void write_html_link (FILE *fp, GString *linkdest, GString *linkname)
+{
+  fprintf (fp, "<a href=\"%s\">%s</a>", linkdest->str, linkname->str);
+}
+
+void write_html_ul_start (FILE *fp)
+{
+  fprintf (fp, "<ul>");
+}
+
+void write_html_ul_end (FILE *fp)
+{
+  fprintf (fp, "</ul>");
+}
+
+void write_html_li_start (FILE *fp)
+{
+  fprintf (fp, "<li>");
+}
+
+void write_html_li_end (FILE *fp)
+{
+  fprintf (fp, "</li>");
+}
+
+void write_html_table_start (FILE *fp)
+{
+  fprintf (fp, "<table cellpadding=\"2\" cellspacing=\"2\" border=\"1\" width=\"75%%\">");
+  fprintf (fp, "<tbody>");
+}
+
+void write_html_table_end (FILE *fp)
+{
+  fprintf (fp, "</table>");
+}
+
+void write_html_td_start (FILE *fp)
+{
+  fprintf (fp, "<td>");
+}
+
+void write_html_td_end (FILE *fp)
+{
+  fprintf (fp, "</td>");
+}
+
+void write_html_tr_start (FILE *fp)
+{
+  fprintf (fp, "<tr>");
+}
+
+void write_html_tr_end (FILE *fp)
+{
+  fprintf (fp, "</tr>");
+}
+
+void write_html_center_start (FILE *fp)
+{
+  fprintf (fp, "<center>");
+}
+
+void write_html_center_end (FILE *fp)
+{
+  fprintf (fp, "</center>");
+}
+
+void write_html_image (FILE *fp, GString *imgfile, GString *alt)
+{
+  if (strcmp (imgfile->str, "")!=0)
+    {
+      fprintf (fp, "<img src=\"%s\" alt=\"%s\">", imgfile->str, alt->str);
+    }
+}
+
+gchar *generate_subdirname (GString *dirname, TComic *TempComic)
+{
+  GString *returnme;
+  int i;
+
+  returnme = g_string_new (NULL);
+
+  g_string_append (returnme, dirname->str);
+
+  /* sanitize string  */
+  /* g_string_append (returnme, TempComic->CTitle->str); */
+  for (i=0; i<strlen(TempComic->CTitle->str); i++)
+    {
+      if (TempComic->CTitle->str[i] == ':' ||
+	  TempComic->CTitle->str[i] == '/' ||
+	  TempComic->CTitle->str[i] == '\\' ||
+	  TempComic->CTitle->str[i] == ';' ||
+	  TempComic->CTitle->str[i] == '?' ||
+	  TempComic->CTitle->str[i] == '&' ||
+	  TempComic->CTitle->str[i] == ' ')
+	{
+	  g_string_append_c (returnme, '_');
+	}
+      else
+	{
+	  g_string_append_c (returnme, TempComic->CTitle->str[i]);
+	}
+    }
+  g_string_append (returnme, "_");
+  g_string_append (returnme, itoa (TempComic->UID));
+
+  return returnme->str;
+}
+
+gchar *generate_comic_filename (GString *dirname, TComic *TempComic)
+{
+  /* The filename is generated from the subdirname by adding .html */
+  GString *returnme;
+
+  returnme = g_string_new (generate_subdirname (dirname, TempComic));
+
+  g_string_append (returnme, ".html");
+
+  return returnme->str;
+}
+
+gchar *generate_issue_filename (GString *dirname, TIssue *TempIssue)
+{
+  GString *returnme;
+  int i;
+
+  returnme = g_string_new (NULL);
+
+  g_string_append (returnme, dirname->str);
+  g_string_append (returnme, "/");
+
+  /* sanitize string  */
+  /* g_string_append (returnme, TempIssue->IssueNo->Str); */
+  for (i=0; i<strlen(TempIssue->IssueNo->str); i++)
+    {
+      if (TempIssue->IssueNo->str[i] == ':' ||
+	  TempIssue->IssueNo->str[i] == '/' ||
+	  TempIssue->IssueNo->str[i] == '\\' ||
+	  TempIssue->IssueNo->str[i] == ';' ||
+	  TempIssue->IssueNo->str[i] == '?' ||
+	  TempIssue->IssueNo->str[i] == '&' ||
+	  TempIssue->IssueNo->str[i] == ' ')
+	{
+	  g_string_append_c (returnme, '_');
+	}
+      else
+	{
+	  g_string_append_c (returnme, TempIssue->IssueNo->str[i]);
+	}
+    }
+  g_string_append (returnme, "_");
+  g_string_append (returnme, itoa (TempIssue->UID));
+  g_string_append (returnme, ".html");
+
+  return returnme->str;
+}
+
+int ExportToHTMLFile(GList *clist, GString *dirname)
+{
+  TComic *TempComic;
+  TIssue *TempIssue;
+  guint32 i,j,k,l,m,n;
+  GString *String, *indexname, *fname, *dname;
+  FILE *indexfp = NULL;
+  FILE *ifp  = NULL;
+  FILE *cfp = NULL;
+
+  indexname = g_string_new (dirname->str);
+  g_string_append (indexname, "index.html");
+  indexfp = fopen (indexname->str, "w");
+  if (indexfp == NULL)
+    {
+      /* Unable to open file */
+      return 1;
+    }
+
+  j=g_list_length(clist);
+
+  write_html_preamble (indexfp, "My Comics");
+  write_html_h1_start (indexfp);
+  fprintf(indexfp, "%i comic title(s)\n\n\n\n", j);
+  write_html_h1_end (indexfp);
+  write_html_br (indexfp);
+  fprintf (indexfp, "\n");
+
+
+  fname = g_string_new (NULL);
+  dname = g_string_new (NULL);
+
+  for(i=0;i<j;i++)
+    {
+      TempComic = (g_list_nth(clist,i))->data;
+
+      /* Construct unique filename */
+      g_string_assign (fname, "");
+      g_string_append (fname, generate_comic_filename (dirname, TempComic));
+      
+      /* open file */
+      cfp = fopen (fname->str, "w");
+      if (cfp == NULL)
+	{
+	  /* Unable to open file */
+	  return 1;
+	}
+    
+      write_html_preamble (cfp, TempComic->CTitle->str);
+
+      String = g_string_new (TempComic->CTitle->str);
+      g_string_append (String, " (Vol. ");
+      g_string_append (String, TempComic->CVolume->str);
+      g_string_append (String, ")");
+      write_html_link (indexfp, fname, String);
+      write_html_br (indexfp);
+      fprintf (indexfp, "\n");
+
+      write_html_h1_start (cfp);
+      fprintf(cfp, "%s", TempComic->CTitle->str);
+      if(TempComic->IsLink)
+	fprintf(cfp, " (is a link to %s)\n", TempComic->LinkFile->str);
+      else
+	fprintf(cfp, " (Vol. %s, %s, %s)\n", TempComic->CVolume->str,
+		TempComic->CType->str, TempComic->CPublisher->str);
+      write_html_h1_end (cfp);
+      write_html_br (cfp);
+      fprintf(cfp, "Comment: %s\n", TempComic->Comment->str);
+      write_html_p (cfp);
+
+      if(TempComic->IssueNumbers!=NULL)
+	{
+	  write_html_h2_start (cfp);
+	  fprintf(cfp, "%i issue(s)\n", l=g_list_length(TempComic->IssueNumbers));
+	  write_html_h2_end (cfp);
+	  write_html_br (cfp);
+
+	  g_string_assign (dname, "");
+	  g_string_append (dname, generate_subdirname (dirname, TempComic));
+	  mkdir (dname->str, 0755); /* read/write/execute by user, read/execute by anyone else - umask permitting */
+      
+	  for(k=0;k<l;k++)
+	    {
+	      TempIssue = (g_list_nth(TempComic->IssueNumbers,k))->data;
+
+	      g_string_assign (fname, "");
+	      g_string_append (fname, generate_issue_filename (dname, TempIssue));
+
+
+	      /* open file */
+	      ifp = fopen (fname->str, "w");
+	      if (ifp == NULL)
+		{
+		  /* Unable to open file */
+		  return 1;
+		}
+
+
+	      String = g_string_new ("#");
+	      g_string_append (String, TempIssue->IssueNo->str);
+	      write_html_link (cfp, fname, String);
+	      write_html_br (cfp);
+	      fprintf (cfp, "\n");
+
+
+	      g_string_assign (String, TempComic->CTitle->str);
+	      g_string_append (String, " - #");
+	      g_string_append (String, TempIssue->IssueNo->str);
+	      write_html_preamble (ifp, String->str);
+    
+	      write_html_center_start (ifp);
+	      write_html_h3_start (ifp);
+	      fprintf(ifp, "  Issue #%s:\n", TempIssue->IssueNo->str);
+	      write_html_h3_end (ifp);
+
+	      write_html_image (ifp, TempIssue->Image, TempIssue->IssueNo);
+
+	      write_html_table_start (ifp);
+
+	      write_html_tr_start (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "Writer(s)");
+	      write_html_td_end (ifp);
+	      write_html_td_start (ifp);
+	      n=g_list_length(TempIssue->Writer);
+	      for(m=0;m<n;m++)
+		{ 
+		  String = g_list_nth(TempIssue->Writer, m)->data;
+		  fprintf(ifp, "      %s\n", String->str);
+		  write_html_br (ifp);
+		}
+	      write_html_td_end (ifp);
+	      write_html_tr_end (ifp);
+
+	      write_html_tr_start (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "Penciller(s)");
+	      write_html_td_end (ifp);
+	      write_html_td_start (ifp);
+	      n=g_list_length(TempIssue->Penciller);
+	      for(m=0;m<n;m++)
+		{ 
+		  String = g_list_nth(TempIssue->Penciller, m)->data;
+		  fprintf(ifp, "      %s\n", String->str);
+		  write_html_br (ifp);
+		}
+	      write_html_td_end (ifp);
+	      write_html_tr_end (ifp);
+	      
+	      write_html_tr_start (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "Inker(s)");
+	      write_html_td_end (ifp);
+	      write_html_td_start (ifp);
+	      n=g_list_length(TempIssue->Inker);
+	      for(m=0;m<n;m++)
+		{ 
+		  String = g_list_nth(TempIssue->Inker, m)->data;
+		  fprintf(ifp, "      %s\n", String->str);
+		  write_html_br (ifp);
+		}
+	      write_html_td_end (ifp);
+	      write_html_tr_end (ifp);
+
+	      write_html_tr_start (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, " StoryTitle(s)");
+	      write_html_td_end (ifp);
+	      write_html_td_start (ifp);
+	      n=g_list_length(TempIssue->Title);
+	      for(m=0;m<n;m++)
+		{ 
+		  String = g_list_nth(TempIssue->Title, m)->data;
+		  fprintf(ifp, "      %s\n", String->str);
+		  write_html_br (ifp);
+		}
+	      write_html_td_end (ifp);
+	      write_html_tr_end (ifp);
+
+	      write_html_tr_start (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "Location");
+	      write_html_td_end (ifp);
+	      write_html_td_start (ifp);
+	      fprintf (ifp, "%s", TempIssue->Location->str);
+	      write_html_td_end (ifp);
+	      write_html_tr_end (ifp);
+
+
+	      write_html_tr_start (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "CoverPrice");
+	      write_html_td_end (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "%s", TempIssue->Price->str);
+	      write_html_td_end (ifp);
+	      write_html_tr_end (ifp);
+	      
+	      write_html_tr_start (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "Current Price");
+	      write_html_td_end (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "%s", TempIssue->CurrentPrice->str);
+	      write_html_td_end (ifp);
+	      write_html_tr_end (ifp);
+
+	      write_html_tr_start (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "Purchase Price");
+	      write_html_td_end (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "%s", TempIssue->PurchasePrice->str);
+	      write_html_td_end (ifp);
+	      write_html_tr_end (ifp);
+
+	      write_html_tr_start (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "Condition");
+	      write_html_td_end (ifp);
+	      write_html_td_start (ifp);
+
+	      switch (TempIssue->Condition)
+		{
+		case Mint:
+		  fprintf(ifp, "Mint\n");
+		  break;
+		case NearMint:      
+		  fprintf(ifp, "NearMint\n");
+		  break;
+		case VeryFine: 
+		  fprintf(ifp, "VeryFine\n");
+		  break;
+		case FineVeryFine: 
+		  fprintf(ifp, "FineVeryFine\n");
+		  break;
+		case Fine: 
+		  fprintf(ifp, "Fine\n");
+		  break;
+		case VeryGood:  
+		  fprintf(ifp, "VeryGood\n");
+		  break;
+		case Good:  
+		  fprintf(ifp, "Good\n");
+		  break;
+		case Fair:  
+		  fprintf(ifp, "Fair\n");
+		  break;
+		case Poor: 
+		  fprintf(ifp, "Poor\n");
+		  break;
+		default: 
+		  fprintf(ifp, "Unknown\n");
+		  break;
+		}
+	      write_html_td_end (ifp);
+	      write_html_tr_end (ifp);
+
+	      write_html_tr_start (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "Comment");
+	      write_html_td_end (ifp);
+	      write_html_td_start (ifp);
+	      fprintf(ifp, "%s", TempIssue->Comment->str);
+	      write_html_td_end (ifp);
+	      write_html_tr_end (ifp);
+
+	      write_html_table_end (ifp);
+	      write_html_center_end (ifp);
+
+	      write_html_end (ifp);
+	      fclose(ifp);
+	    }
+	}
+      else
+	{
+	  write_html_ul_start (cfp);
+	  write_html_li_start (cfp);
+	  fprintf(ifp, "no issues\n");
+	  write_html_li_end (cfp);
+	  write_html_ul_end (cfp);
+	}
+      write_html_end (cfp);
+      fclose(cfp);
+    }
+  
+  write_html_end (indexfp);
+  fclose(indexfp);
+  return 0;
 }
