@@ -26,4 +26,5 @@ typedef enum{CO15_16a, CO16b_CO16c, CO17, CO17a, CO17b, CO17c_CO17d,
 GList *LoadFile(GList *ComicList, FILE *fp);
 void SaveFile(GList *ComicList, FILE *fp);
 TFileType FileCheck(FILE *fp);
-void ExportToASCIIFile(GList *clist, FILE *fp);
+int ExportToASCIIFile(GList *clist, GString *fname);
+int ExportToHTMLFile(GList *clist, GString *fname);
